--[[
Spezialisierung zum rendern der Anzahl von verbleibenden Ballen für Folie und Garn auf den Bildschirm.

Author:	Ifko[nator]
Datum:	03.10.2015

Version:	v3.0

History:	v1.0 @ 18.09.2015 - initial implementation
			v2.0 @ 29.09.2015 - the timing of the net wrap consumption has been adjusted, little bit dirty but it works
							  -	set correct rotation for the bale in the bale chamber
							  - bales will now added to the ingame stats, also the already pressed bales
			v3.0 @ 03.10.2015 - set correct rotation for the middle net roll
							  - animated pickup roll when baler is filling
							  - bug fix for the case, the main chamber door is opening and the wrapper table is not rotated forwards
							  - bug fix for load the wrong bale size, little bit dirty but it works
]]

KroneUltimaExtension = {};

local kroneUltimaExtension_directory = g_currentModDirectory;

function KroneUltimaExtension.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Ultima, specializations);
end;

function KroneUltimaExtension:load(xmlFile)
	if self.baleCounterClackBackGroundImg ~= nil then
		self.hudPosYStart = 0.295;
		self.hudPosYOffSet = 0.168;
	else
		self.hudPosYStart = 0.29;
		self.hudPosYOffSet = 0.173;
	end;
	
	self.hudPosY = self.hudPosYStart;
	self.hudWidth = 0.0335;
	self.hudHeight = 0.045;
	
	self.hudFoilPosX = 0.826;
	self.hudNetPosX = 0.915;
	self.hudBackgroundPosX = 0.855;
	
	self.hudBackgroundWidth = 0.15;
	
	self.hudOverlayFoilFilename = Utils.getFilename("baleFoil.dds", kroneUltimaExtension_directory);
	self.hudOverlayFoilCamOff = Overlay:new("hudOverlayFoilCamOff", self.hudOverlayFoilFilename, self.hudFoilPosX, self.hudPosY, self.hudWidth, self.hudHeight);
	self.hudOverlayFoilCamOn = Overlay:new("hudOverlayFoilCamOn", self.hudOverlayFoilFilename, self.hudFoilPosX, self.hudPosY + self.hudPosYOffSet, self.hudWidth, self.hudHeight);
	
	self.hudOverlayNetFilename = Utils.getFilename("baleNet.dds", kroneUltimaExtension_directory);
	self.hudOverlayNetCamOff = Overlay:new("hudOverlayNetCamOff", self.hudOverlayNetFilename, self.hudNetPosX, self.hudPosY, self.hudWidth, self.hudHeight);
	self.hudOverlayNetCamOn = Overlay:new("hudOverlayNetCamOn", self.hudOverlayNetFilename, self.hudNetPosX, self.hudPosY + self.hudPosYOffSet, self.hudWidth, self.hudHeight);
	
	self.hudOverlayBackgroundFilename = Utils.getFilename("background.dds", kroneUltimaExtension_directory);
	self.hudOverlayBackgroundCamOff = Overlay:new("hudOverlayBackgroundCamOff", self.hudOverlayBackgroundFilename, self.hudBackgroundPosX, self.hudPosY, self.hudBackgroundWidth, self.hudHeight);
	self.hudOverlayBackgroundCamOn = Overlay:new("hudOverlayBackgroundCamOn", self.hudOverlayBackgroundFilename, self.hudBackgroundPosX, self.hudPosY + self.hudPosYOffSet, self.hudBackgroundWidth, self.hudHeight);
	
	self.netBindingStart = Utils.appendedFunction(self.netBindingStart, KroneUltimaExtension.netBindingStart);
	self.dropBaleStart = Utils.appendedFunction(self.dropBaleStart, KroneUltimaExtension.dropBaleStart);
	
	self.wrapperDropBale = Utils.overwrittenFunction(self.wrapperDropBale, KroneUltimaExtension.wrapperDropBale);
	self.setFillLevel = Utils.overwrittenFunction(self.setFillLevel, KroneUltimaExtension.setFillLevel);
	
	if self.balerBaleConductNode ~= nil then
		--print("--## set correct rotation for the baler bale conduct node.");
		setRotation(self.balerBaleConductNode, math.rad(-180), math.rad(90), math.rad(-180));
	end;
	
	local middleNetRoll = Utils.indexToObject(self.components, "0>0|4|0|0");
	if middleNetRoll ~= nil then
		--print("--## set correct rotation for the middle net roll.");
		setRotation(middleNetRoll, math.rad(180), 0, 0);
	end;
	
	for _, workArea in pairs(self.workAreas) do
		--print("--## set Tanslation for workArea start");
		setTranslation(workArea.start, 1.5, 0, 0);
		--print("--## set Tanslation for workArea width");
		setTranslation(workArea.width, -1.5, 0, 0);
		--print("--## set Tanslation for workArea height");
		setTranslation(workArea.height, 1.5, 0, -0.75);
	end;
	
	self.numLeftBalesNet = 0;
	
	self.pickupRoll = Utils.indexToObject(self.components, "0>0|8|0|7|0");
	
	self.previousLevel = self.fillLevel;
	self.previousPpcLevel = self.ppcFillLevel;
	self.pickupIsAktiv = false;
	self.sameFillLevelTime = 0;
	self.rollSpeed = 0;
	
	self.wrapper = Utils.indexToObject(self.components, "0>0|3|6");
end;

function KroneUltimaExtension:setFillLevel(oldFunc, fillLevel, fillType, force, fillSourceStruct) 
	--print("--## using: 'KroneUltimaExtension:setFillLevel'.");
	
	--## dirty hack do stop bale load at Savegame loading ..
	if hasBaleSizeBugFixed == nil then
		hasBaleSizeBugFixed = true;
		self.fillLevel = fillLevel;
		--print("Bale size Bug Fix successful! :)");
	else
		oldFunc(self, fillLevel, fillType, force, fillSourceStruct);
		if self.currentBalerState == 1 or self.currentBalerState == 4 then
			if self.balerCurrentBale == nil and fillLevel > 0 and fillType ~= Fillable.FILLTYPE_UNKNOWN then
				-- balerCreateBale is not synched at all
				self.balerCurrentBale = self:balerCreateBale(self.baleSizeIndex, fillType);
				self.capacity = self.balerCurrentBale.fillLevel;
				self.chamberFillAnim:play();
			end;
			self.chamberFillAnim:setRelativeTime(fillLevel/self.maxChamberFillLevel);
		end;
	end;
end;

function KroneUltimaExtension:netBindingStart()
	--print("--## using: 'KroneUltimaExtension:netBindingStart'.");
	
	--## dirty hack do stop the count down of the net wrap bale count when the net bindig starts .. (simply add the net to the net role that was removed from the Ultima.lua)
	if self.isServer then
		local hasLastBaleDropped;
		local netNeeded = self.numNetWraps * math.pi * self.baleSizes[self.baleSizeIndex].size
		local allowsetFillStateUp = self.netRoleTop.length > 0 or (self.numLeftBalesNet == 1 and not hasLastBaleDropped);
		
		if allowsetFillStateUp then
			self:setNetRoleFillState(self.netRoleTop.length + netNeeded);
		end;
	
		if self.numLeftBalesNet == 1 and not hasLastBaleDropped then
			--## allow drop the last Bale when the net role is empty after net binding this bale ..
			hasLastBaleDropped = true;
		end;
	end;
end;

function KroneUltimaExtension:dropBaleStart()
	--print("--## using: 'KroneUltimaExtension:dropBaleStart'.");
	if self.isServer then
		--## now, when the net binding is finished, the counter for the net wrap bales will be set down ..
		local netNeeded = self.baleSizes[self.balerCurrentBale.sizeIndex].size * math.pi * self.numNetWraps;
		self:setNetRoleFillState(self.netRoleTop.length - netNeeded);
		
		--## rotate wrapper table forwards, if the main chamber door is opening and the wrapper table is not forward rotatet ..
		if self.wrapper ~= nil then
			local x, _, _ = getRotation(self.wrapper);
			local hasRotatet;
		
			if x <= 0 then
				if not hasRotatet then
					--print("--## rotating wrapper table now! Bug Fix successful! :)");
					hasRotatet = true;
					self:setWrapperState(2);
				end;
			end;
		end;
	end;
end;

function KroneUltimaExtension:wrapperDropBale()
	--print("--## using: 'KroneUltimaExtension:wrapperDropBale'.");
	
	if self.isServer then
		removeJoint(self.wrapperCurrentBale.baleJointIndex);
		g_currentMission:addItemToSave(self.wrapperCurrentBale.baleObject);
		self.wrapperCurrentBale.baleJointIndex = nil;
		self.wrapperCurrentBale.sizeIndex = nil;
		self.wrapperCurrentBale.fillType = nil;
		self.wrapperCurrentBale = nil;
		self:setBaleCounter(self.baleCountDay + 1, self.baleCountLifetime + 1);
		g_currentMission.missionStats:updateStats("baleCount", 1);
	end;
end;

function KroneUltimaExtension:update(dt)
	if self:getIsActive() and self.attacherVehicle ~= nil and self.pickupRoll ~= nil and self.isServer then
		if self.attacherVehicle.lastSpeedReal > 0 then
			if self.previousLevel ~= self.fillLevel or self.previousPpcLevel ~= self.ppcFillLevel then
				if self.previousLevel < self.fillLevel or self.previousPpcLevel < self.ppcFillLevel then
					self.pickupIsAktiv = true;
				else
					self.pickupIsAktiv = false;
				end;
				
				self.previousLevel = self.fillLevel;
				self.previousPpcLevel = self.ppcFillLevel;
				self.sameFillLevelTime = 0;
			else
				self.sameFillLevelTime = self.sameFillLevelTime + 1;
					
				if self.sameFillLevelTime < 10 and self.attacherVehicle.lastSpeedReal > 0 then					
					self.pickupIsAktiv = true;
				else
					self.pickupIsAktiv = false;
				end;
			end;
		else
			self.pickupIsAktiv = false;
			self.sameFillLevelTime = 11;
		end;
		
		--renderText(0.5, 0.5, 0.04, "sameFillLevelTime: " .. self.sameFillLevelTime);
		
		local x, _, _ = getRotation(self.pickupRoll);
		
		if self.attacherVehicle.lastSpeedReal > 0.1/3600 then
			if self.pickupIsAktiv then
				if self.rollSpeed < 0.6 then
					self.rollSpeed = self.rollSpeed + 0.018;
				end;
			else
				if self.rollSpeed > 0 then
					self.rollSpeed = self.rollSpeed - 0.005;
				end;
			end;
		else
			if self.rollSpeed > 0 then
				self.rollSpeed = self.rollSpeed - 0.018;
			end;
		end;
			
		--renderText(0.5, 0.46, 0.04, "speed: " .. self.rollSpeed);
		
		if self.rollSpeed < 0 then
			self.rollSpeed = 0;
		end;
		
		x = x + self.rollSpeed;
		
		setRotation(self.pickupRoll, x, 0, 0);
	end;
end;

function KroneUltimaExtension:draw()
	if self:getIsActive() then
		if self.gui ~= nil and self.gui.baseElement ~= nil then
			if self.gui.baseElement.visibility then
				self.hudOverlayBackgroundCamOn:render();
				self.hudOverlayFoilCamOn:render();
				self.hudOverlayNetCamOn:render();
			
				self.hudPosY = 0.465;
			else
				self.hudOverlayBackgroundCamOff:render();
				self.hudOverlayFoilCamOff:render();
				self.hudOverlayNetCamOff:render();
				
				self.hudPosY = self.hudPosYStart;
			end;
		else
			self.hudPosY = self.hudPosYStart;
		end;
	
		setTextAlignment(RenderText.ALIGN_CENTER);
		
		local foilLength = self.wrapperFoilHolders[1].remainingFoilLength * 2;
		local foilPerBale = self.baseFoilUsePerBale * self.baleSizes[self.baleSizeIndex].size;		
		local numLeftBalesFoil = math.ceil(foilLength / foilPerBale);
		
		if numLeftBalesFoil <= 5 then
			setTextColor(0.95, 0, 0, 1);
		else
			setTextColor(1, 1, 1, 1);
		end;
			
		renderText(self.hudFoilPosX + 0.06, self.hudPosY + 0.014, 0.028, "" .. numLeftBalesFoil);

		local netPerBale = self.numNetWraps * math.pi * self.baleSizes[self.baleSizeIndex].size;
		
		self.numLeftBalesNet = math.ceil(self.netRoleTop.length / netPerBale);	
		
		if self.numLeftBalesNet <= 5 then
			setTextColor(0.95, 0, 0, 1);
		else
			setTextColor(1, 1, 1, 1);
		end;
		
		renderText(self.hudNetPosX + 0.06, self.hudPosY + 0.014, 0.028, "" .. self.numLeftBalesNet);
	end;
	
	setTextAlignment(RenderText.ALIGN_LEFT);
end;

function KroneUltimaExtension:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		local addedBalesToStats = getXMLString(xmlFile, key .. "#addedBalesToStats");
		local baleCountLifetime = Utils.getNoNil(getXMLInt(xmlFile, key .. "#baleCountLifetime"), 0);
		
		if baleCountLifetime > 0 and addedBalesToStats == nil then
			print("[INFO from the KroneUltimaExtension.lua]: " .. baleCountLifetime .. " bales has been added to the bale count statistics!");
			g_currentMission.missionStats:updateStats("baleCount", baleCountLifetime);
		end;
	end;

	return BaseMission.VEHICLE_LOAD_OK;
end;

function KroneUltimaExtension:getSaveAttributesAndNodes(nodeIdent)
    local attributes = 'addedBalesToStats="done"';

	return attributes, nil;
end;

function KroneUltimaExtension:delete()end;
function KroneUltimaExtension:mouseEvent(posX, posY, isDown, isUp, button)end;
function KroneUltimaExtension:keyEvent(unicode, sym, modifier, isDown)end;